package it.polito.bigdata.spark;

import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.sql.types.DataTypes;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;


public class SparkDriver {
	
	public static void main(String[] args) {

		// The following two lines are used to switch off some verbose log messages
		Logger.getLogger("org").setLevel(Level.OFF);
		Logger.getLogger("akka").setLevel(Level.OFF);


		// String companiesPath; Useless for this program
		String dataCenterPath, dailyPwrConsPath;
		String outputPath1, outputPath2;

		// companiesPath = args[0]; Useless for this program
		dataCenterPath = args[1];
		dailyPwrConsPath = args[2];
		outputPath1 = args[3];
		outputPath2 = args[4];		


		// Create a Spark Session object and set the name of the application
		SparkSession ss = SparkSession.builder().appName("Exam20220916 Spark")
									.master("local").getOrCreate();

		// Define the dataframes associated with the input files
		// companiesPath is useless for this application

		Dataset<Row> dataCenterDF = ss.read().format("csv")
				.option("header", false).option("inferSchema", true)
				.load(dataCenterPath).cache()
				.withColumnRenamed("_c0", "CodDC")
				.withColumnRenamed("_c1", "CodC")
				.withColumnRenamed("_c2", "City")
				.withColumnRenamed("_c3", "Country")
				.withColumnRenamed("_c4", "Continent");
				
		// Register the temporary table DataCenters 
		dataCenterDF.createOrReplaceTempView("DataCenters");
	

		Dataset<Row> pwrConsDF = ss.read().format("csv")
				.option("header", false).option("inferSchema", true)
				.load(dailyPwrConsPath).cache()
				.withColumnRenamed("_c0", "CodDC")
				.withColumnRenamed("_c1", "Date")
				.withColumnRenamed("_c2", "kWh");

		// Register the temporary table DailyPwrCons
		pwrConsDF.createOrReplaceTempView("DailyPwrCons");



		// Part 1
		// Select the dates associated with high power consumption
		// and count the number of data centers with high power consumption for each of those dates.
		// Select the dates with at least 90% of the data centers consuming at least 1000 kWh each.
		Dataset<Row> res1SelectedDateDF = 
				ss.sql("SELECT Date "
			  		+ "FROM DailyPwrCons "
					+ "WHERE kWh>=1000 "
					+ "GROUP BY Date "
					+ "HAVING COUNT(*)>=(SELECT COUNT(*)*0.9 "
					+ "					 FROM DataCenters)");

		res1SelectedDateDF.write()
				.format("csv")
				.option("header", false)
				.save(outputPath1); 


		// Part 2

		// Register a function that given a month returns the year
		ss.udf().register("EXTRACTYEAR", 
			(String month) -> Integer.parseInt(month.split("/")[0]),
				DataTypes.IntegerType);


		// Consider the power consumptions and keep only the entries related to year 2021
		// and compute the power consumption for the entire year for each data center
		Dataset<Row> yearlyPwrCons2021DF = 
			ss.sql("SELECT CodDC, SUM(kWh) AS Cons2021 "
			  	+ "FROM DailyPwrCons "
				+ "WHERE EXTRACTYEAR(Date)=2021 "
				+ "GROUP BY CodDC ");

		// Register the temporary table yearlyPwrCons2021
		yearlyPwrCons2021DF.createOrReplaceTempView("yearlyPwrCons2021");



		// Compute for each continent the number of data centers and the 
		// the avg power consumption per data center in year 2021
		Dataset<Row> numDCandAvgPwrConsDF = 
			ss.sql("SELECT Continent, COUNT(*) AS NumDataCenters, AVG(Cons2021) AS AvgCons2021PerDC "
			  	+ "FROM yearlyPwrCons2021, DataCenters "
				+ "WHERE yearlyPwrCons2021.CodDC=DataCenters.CodDC "
				+ "GROUP BY Continent").cache();

		// Register the temporary table numDCandAvgPwrCons
		numDCandAvgPwrConsDF.createOrReplaceTempView("numDCandAvgPwrCons");




		// Compute the maximum number of data centers and the maximum avg consumption 
		// (see the content of the table function in the following query) and
		// select only the continent(s) satisfying the two constraints

		Dataset<Row> res2SelectedContinentsDF = 
				ss.sql("SELECT Continent "
					+ "FROM numDCandAvgPwrCons, (SELECT Max(NumDataCenters) AS MaxDCs, "
					+ "									Max(AvgCons2021PerDC) AS MaxAvgCons2021PerDC "
					+ "							 FROM numDCandAvgPwrCons) MaxTable "
					+ "WHERE numDCandAvgPwrCons.NumDataCenters=MaxTable.MaxDCs "
					+ "  AND numDCandAvgPwrCons.AvgCons2021PerDC=MaxTable.MaxAvgCons2021PerDC");



		res2SelectedContinentsDF.write()
				.format("csv")
				.option("header", false)
				.save(outputPath2); 

		// Close the Spark session
		ss.stop();
	}
}
